--******************************************************************************************************************************
--* Licensed Materials - Property of IBM                                                                                       *
--* (c) Copyright IBM Corp. 2008, 2009 All Rights Reserved.                                                                    *
--*                                                                                                                            *
--*US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp*
--******************************************************************************************************************************

SET CURRENT SCHEMA $STAR_SCHEMA@

CREATE FUNCTION SELECT_CAL_MESSAGE (CAL_MSG_CODE VARCHAR(15))
RETURNS VARCHAR(255)
NO EXTERNAL ACTION 
READS SQL DATA
BEGIN ATOMIC

	DECLARE v_msg_code VARCHAR(255);
	SET v_msg_code=RTRIM(CAL_MSG_CODE);
	
	RETURN SELECT COALESCE(CAL_ERR_MESSAGE, 'CRREW0000E') FROM CAL_MESSAGES WHERE RTRIM(CAL_MESSAGE_CODE)=v_msg_code;       

END@

	
CREATE PROCEDURE POPULATE_D_DATE (IN START_YEAR INTEGER, IN END_YEAR INTEGER, SCHEMA_NAME CHAR(8))
RESULT SETS 1
LANGUAGE SQL     
BEGIN
 
		DECLARE SQLSTATE CHAR(5) DEFAULT '00000';         
		DECLARE SQLCODE INTEGER DEFAULT 0;
		DECLARE MESSAGE_TEXT VARCHAR(100);
		DECLARE RETVAL INTEGER DEFAULT 0; 
		DECLARE v_sqlstate CHAR(5) DEFAULT '00000';
    	DECLARE v_rise INTEGER DEFAULT 0;
    	DECLARE v_errmsg VARCHAR(100) DEFAULT '';
    	

		DECLARE v_date_id INTEGER;
		DECLARE v_date DATE;  
		DECLARE v_year_str CHAR(4);                         
		DECLARE v_year INTEGER DEFAULT 0; 
		DECLARE v_previous_year INTEGER DEFAULT 0;             
		DECLARE v_quarter_of_year INTEGER DEFAULT 0;  
		DECLARE v_month_of_year INTEGER DEFAULT 0;    
		DECLARE v_month_name VARCHAR(20);                
		DECLARE v_week_of_year INTEGER DEFAULT 0;  
		DECLARE v_week_of_year_iso INTEGER DEFAULT 0;     
		DECLARE v_quarter_no INTEGER DEFAULT 0;       
		DECLARE v_month_no INTEGER DEFAULT 0;         
		DECLARE v_week_no INTEGER DEFAULT 0;   
		DECLARE v_week_no_iso INTEGER DEFAULT 0;
		DECLARE v_week_of_month INTEGER DEFAULT 0;
		DECLARE v_week_of_month_iso INTEGER DEFAULT 0;
		DECLARE v_day_of_week INTEGER DEFAULT 0;
		DECLARE v_day_of_week_iso INTEGER DEFAULT 0;
		DECLARE v_day_of_month INTEGER DEFAULT 0;
		DECLARE v_day_of_year INTEGER DEFAULT 0;
		DECLARE v_day_name VARCHAR(20);
		DECLARE v_previous_quarter INTEGER DEFAULT 0; 
		DECLARE v_previous_month INTEGER DEFAULT 0;   
		DECLARE v_previous_week INTEGER DEFAULT 0;   
		DECLARE v_previous_week_iso INTEGER DEFAULT 0;  
		--DECLARE v_calendar_type INTEGER;
		--DECLARE v_calendar_type_c CHAR(1);
		DECLARE v_schema_name CHAR(8);
		DECLARE stmt VARCHAR(1000) DEFAULT ''; 
		DECLARE err_stmt VARCHAR(1000) DEFAULT '';
		 
		DECLARE v_print VARCHAR(100); 
		DECLARE v_min_date DATE;
		DECLARE v_max_date DATE;
		
		-- Declare condition handlers
	  DECLARE EXIT HANDLER FOR SQLEXCEPTION                               
	     BEGIN
	        -- Would like to have used this method but custom MESSAGE_TEXT is limited to 70 chars
	     		--GET DIAGNOSTICS EXCEPTION 1 v_token = MESSAGE_TEXT;
	     		--SET v_errmsg= 'sqlstate:' || SQLSTATE || ' sqlcode:' ||CHAR(SQLCODE) || ' message:' || v_token; 
	        --RESIGNAL SQLSTATE '70000' SET MESSAGE_TEXT=v_errmsg;   
	        RESIGNAL;                                               
	     END;
		
		 DECLARE EXIT HANDLER FOR SQLSTATE '70001'                              
			BEGIN    
				SET err_stmt='VALUES (' || v_schema_name || '.SELECT_CAL_MESSAGE(''CRREW4586E''))';
				PREPARE s_err1 FROM err_stmt;
				BEGIN
					DECLARE CE1 CURSOR FOR s_err1;
					OPEN CE1;
					FETCH CE1 INTO v_errmsg;
					CLOSE CE1;
				END;                                                                                                                       
				RESIGNAL SQLSTATE '70001' SET MESSAGE_TEXT = v_errmsg;                                                                 
			END;

		-- stop if sqlstate 42710 table or sequence already exists
	   DECLARE EXIT HANDLER FOR SQLSTATE '42710'
			 BEGIN  
			  IF POSSTR(stmt,'CREATE TABLE')>0 THEN
				 	RESIGNAL SQLSTATE '70002' SET MESSAGE_TEXT = ''; 
				 	ELSE
				 	RESIGNAL SQLSTATE '70003' SET MESSAGE_TEXT = '';  
			 	END IF;                                                            
			 END;
		 
		 DECLARE EXIT HANDLER FOR SQLSTATE '70004'                              
			BEGIN
				SET err_stmt='VALUES (' || v_schema_name || '.SELECT_CAL_MESSAGE(''CRREW4587E''))';
				PREPARE s_err2 FROM err_stmt;
				BEGIN
					DECLARE CE2 CURSOR FOR s_err2;
					OPEN CE2;
					FETCH CE2 INTO v_errmsg;
					CLOSE CE2;
				END;                                                                                                                       
				RESIGNAL SQLSTATE '70004' SET MESSAGE_TEXT = v_errmsg;                                                                 
			END;
		 
		 DECLARE EXIT HANDLER FOR SQLSTATE '70005'                              
			BEGIN
			 	SET err_stmt='VALUES (' || v_schema_name || '.SELECT_CAL_MESSAGE(''CRREW4588E''))';
				PREPARE s_err3 FROM err_stmt;
				BEGIN
					DECLARE CE3 CURSOR FOR s_err3;
					OPEN CE3;
					FETCH CE3 INTO v_errmsg;
					CLOSE CE3;
				END;                                                                                                                         
				RESIGNAL SQLSTATE '70005' SET MESSAGE_TEXT = v_errmsg;                                                                 
			END;
			
			DECLARE EXIT HANDLER FOR SQLSTATE '70006'                              
			BEGIN
			 	SET err_stmt='VALUES (' || v_schema_name || '.SELECT_CAL_MESSAGE(''CRREW4589E''))';
				PREPARE s_err4 FROM err_stmt;
				BEGIN
					DECLARE CE4 CURSOR FOR s_err4;
					OPEN CE4;
					FETCH CE4 INTO v_errmsg;
					CLOSE CE4;
				END;                                                                                                                                 
				RESIGNAL SQLSTATE '70006' SET MESSAGE_TEXT = v_errmsg;                                                                 
			END;
			

		--SET v_calendar_type=CALENDAR_TYPE;
		--SET v_calendar_type_c= CASE WHEN v_calendar_type=1 THEN 'G' WHEN v_calendar_type=2 THEN 'I' ELSE 'G' END;
		
		SET v_schema_name=UPPER(RTRIM(SCHEMA_NAME));
		
		IF (START_YEAR IS NULL) OR (START_YEAR>END_YEAR) THEN
			SIGNAL SQLSTATE '70004';
		END IF;
		
		IF (END_YEAR IS NULL) OR (END_YEAR < START_YEAR) THEN
			SIGNAL SQLSTATE '70005';
		END IF;
		
		--IF v_calendar_type<>1 AND v_calendar_type <> 2 THEN
		--		 SIGNAL SQLSTATE '70001'; 
		--END IF;    
		
		IF (length(rtrim(v_schema_name))=0) THEN
				 SIGNAL SQLSTATE '70006'; 
		END IF;    

		
 
    SET v_year_str=RTRIM(CHAR(START_YEAR));
    SET v_min_date= DATE( v_year_str || '-' || '01-01');
    SET v_year_str=RTRIM(CHAR(END_YEAR));
    SET v_max_date= DATE(v_year_str || '-' || '12-31');
    SET v_previous_year=START_YEAR;
    SET v_week_of_month=1;
    SET v_day_of_month=0;
    
    WHILE(v_min_date<=v_max_date) DO
    
	    	SET v_year=YEAR(v_min_date);
	    	SET v_quarter_of_year=QUARTER(v_min_date);
	    	SET v_month_of_year=MONTH(v_min_date);
	    	SET v_month_name=MONTHNAME(v_min_date);
	    	SET v_week_of_year=WEEK(v_min_date);
	    	SET v_week_of_year_iso=WEEK_ISO(v_min_date);
	    	SET v_quarter_no= CASE WHEN v_quarter_of_year<>v_previous_quarter THEN v_quarter_no+1 ELSE v_quarter_no END;
	    	SET v_month_no= CASE WHEN v_month_of_year<> v_previous_month THEN v_month_no+1 ELSE v_month_no END;
	    	SET v_week_no= CASE WHEN v_week_of_year<>v_previous_week THEN v_week_no+1 ELSE v_week_no END; 
	    	SET v_week_no_iso= CASE WHEN v_week_of_year_iso<>v_previous_week_iso THEN v_week_no_iso+1 ELSE v_week_no_iso END; 
			SET v_day_of_week =DAYOFWEEK(v_min_date);
			SET v_day_of_week_iso =DAYOFWEEK_ISO(v_min_date);
			SET v_day_of_month=CASE WHEN v_month_of_year<> v_previous_month THEN 1 ELSE v_day_of_month+1 END;
			SET v_day_of_year =DAYOFYEAR(v_min_date);
			SET v_day_name=DAYNAME(v_min_date);
			
			IF (v_month_of_year<> v_previous_month ) THEN
					SET v_week_of_month=1;
					IF(v_day_of_week_iso<5) THEN
						SET v_week_of_month_iso=1;
					ELSE SET v_week_of_month_iso=0;
					END IF;
			ELSE
			   IF (v_day_of_week=1) THEN
					SET v_week_of_month=v_week_of_month+1;	
					END IF;
				 IF (v_day_of_week_iso=1) THEN
					SET v_week_of_month_iso=v_week_of_month_iso+1;
	        	 END IF; 
			END IF;
								
    		SET stmt= 'SELECT NEXTVAL FOR ' || v_schema_name || '.SEQ_D_DATE FROM SYSIBM.SYSDUMMY1';
    	
	    	PREPARE s3 FROM stmt;
				BEGIN
					DECLARE C1 CURSOR FOR s3;
					OPEN C1;
					FETCH C1 INTO v_date_id;
					CLOSE C1;
				END;
  	
    		SET stmt='INSERT INTO ' || v_schema_name || '.D_DATE (DATE_ID,PERIOD_START_DATE,DAY_OF_WEEK ,DAY_OF_MONTH,DAY_OF_YEAR,DAY_NAME,WEEK_OF_MONTH ,WEEK_OF_YEAR,WEEK_NO,MONTH_OF_YEAR,MONTH_NAME,MONTH_NO,QUARTER_OF_YEAR,QUARTER_NO,YEAR_OF,DAY_OF_WEEK_ISO,WEEK_OF_MONTH_ISO,WEEK_OF_YEAR_ISO,WEEK_NO_ISO) VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)';
    
      		PREPARE s4 FROM stmt;
      		EXECUTE s4 USING v_date_id,v_min_date,v_day_of_week,v_day_of_month,v_day_of_year,v_day_name,v_week_of_month,v_week_of_year,v_week_no,v_month_of_year,v_month_name,v_month_no,v_quarter_of_year,v_quarter_no,v_year,v_day_of_week_iso,v_week_of_month_iso,v_week_of_year_iso,v_week_no_iso;
      
      		SET v_previous_quarter=v_quarter_of_year;
      		SET v_previous_month=v_month_of_year;
      		SET v_previous_week=v_week_of_year;
      		SET v_previous_week_iso=v_week_of_year_iso;
      		SET v_min_date=v_min_date+ 1 DAYS;
 	END WHILE; 
 	
 	--SET stmt= 'UPDATE CONFIG.SYSCONFIG_INFO SET CHAR_VALUE=''' || v_calendar_type_c || ''',INT_VALUE=' || CHAR(v_calendar_type) || ' WHERE NAME=''CALENDAR_WEEK_TYPE''';
 	--PREPARE s7 FROM stmt;
 	--EXECUTE s7;
 	COMMIT;
END@             

CREATE PROCEDURE RECALCULATE_D_DATE (SCHEMA_NAME CHAR(8))
RESULT SETS 1
LANGUAGE SQL     
BEGIN

		DECLARE v_total_rows INTEGER;
		DECLARE v_week_no INTEGER DEFAULT 0;
		DECLARE v_week_no_iso INTEGER DEFAULT 0;
		DECLARE v_month_no INTEGER DEFAULT 0;
		DECLARE v_quarter_no INTEGER DEFAULT 0;
		DECLARE v_cur_week_no INTEGER DEFAULT 0;
		DECLARE v_cur_week_no_iso INTEGER DEFAULT 0;
		DECLARE v_cur_month_no INTEGER DEFAULT 0;
		DECLARE v_cur_quarter_no INTEGER DEFAULT 0;
		DECLARE v_prev_week_no INTEGER DEFAULT 0;
		DECLARE v_prev_week_no_iso INTEGER DEFAULT 0;
		DECLARE v_prev_month_no INTEGER DEFAULT 0;
		DECLARE v_prev_quarter_no INTEGER DEFAULT 0;
		--DECLARE v_calendar_type INTEGER;
		DECLARE v_schema_name CHAR(8);
		DECLARE stmt VARCHAR(1000) DEFAULT ''; 
		DECLARE stmt2 VARCHAR(1000) DEFAULT '';
		DECLARE v_cur_date DATE;
		DECLARE v_prev_date DATE;
		DECLARE no_rec INTEGER DEFAULT 0;
		
		DECLARE not_found CONDITION FOR SQLSTATE '02000';
 		DECLARE CONTINUE HANDLER FOR not_found
    SET no_rec = 1;
    
    SET v_schema_name=SCHEMA_NAME;

		SET stmt='SELECT PERIOD_START_DATE,WEEK_OF_YEAR,WEEK_OF_YEAR_ISO,MONTH_OF_YEAR,QUARTER_OF_YEAR FROM ' || CHAR(v_schema_name) || '.D_DATE ORDER BY PERIOD_START_DATE';
		PREPARE st1 FROM stmt;
				BEGIN
					DECLARE CS1 CURSOR FOR st1;
					OPEN CS1;
					updt_loop:
					LOOP
					FETCH CS1 INTO v_cur_date,v_cur_week_no,v_cur_week_no_iso,v_cur_month_no,v_cur_quarter_no;
					IF (no_rec=1) THEN
						LEAVE updt_loop;
					END IF;
				  IF (v_cur_week_no<>v_prev_week_no) THEN 
					   SET v_week_no=v_week_no+1;
					   SET v_prev_week_no=v_cur_week_no;
					END IF;
					
					IF (v_cur_week_no_iso<>v_prev_week_no_iso) THEN 
					   SET v_week_no_iso=v_week_no_iso+1;
					   SET v_prev_week_no_iso=v_cur_week_no_iso;
					END IF;
					
					IF (v_cur_month_no<>v_prev_month_no) THEN 
					   SET v_month_no=v_month_no+1;
					   SET v_prev_month_no=v_cur_month_no;
					END IF;
					
					IF (v_cur_quarter_no<>v_prev_quarter_no) THEN 
					   SET v_quarter_no=v_quarter_no+1;
					   SET v_prev_quarter_no=v_cur_quarter_no;
					END IF;
					
					SET stmt2='UPDATE ' || v_schema_name || '.D_DATE SET WEEK_NO=' || CHAR(v_week_no) || ',WEEK_NO_ISO=' ||  CHAR(v_week_no_iso) || ',MONTH_NO=' || CHAR(v_month_no) || ', QUARTER_NO='|| CHAR(v_quarter_no) || ' WHERE PERIOD_START_DATE=''' || CHAR(v_cur_date) || '''';
					PREPARE s9 FROM stmt2;
					EXECUTE s9;
					
					ITERATE updt_loop;
					END LOOP;
					
					CLOSE CS1;
				END;
				
		
	RETURN;

END@



CREATE PROCEDURE EXTEND_D_DATE (IN START_YEAR INTEGER, IN END_YEAR INTEGER, SCHEMA_NAME CHAR(8))
RESULT SETS 1
LANGUAGE SQL     
BEGIN
 
		DECLARE SQLSTATE CHAR(5) DEFAULT '00000';         
		DECLARE SQLCODE INTEGER DEFAULT 0;
		DECLARE MESSAGE_TEXT VARCHAR(100);
		DECLARE RETVAL INTEGER DEFAULT 0; 
		DECLARE v_sqlstate CHAR(5) DEFAULT '00000';
    	DECLARE v_rise INTEGER DEFAULT 0;
    	DECLARE v_errmsg VARCHAR(100) DEFAULT '';
		
		DECLARE v_total_rows INTEGER;
		DECLARE v_day_of_week INTEGER DEFAULT 0;
		DECLARE v_day_name VARCHAR(20);
		--DECLARE v_calendar_type INTEGER;
		DECLARE v_schema_name CHAR(8);
		DECLARE stmt VARCHAR(1000) DEFAULT ''; 
		DECLARE err_stmt VARCHAR(1000) DEFAULT ''; 
		DECLARE stmt2 VARCHAR(1000) DEFAULT '';
		DECLARE v_min_year INTEGER;
		DECLARE v_max_year INTEGER;
		DECLARE v_min_date DATE;
		DECLARE v_max_date DATE;
		
		-- Declare condition handlers
	  DECLARE EXIT HANDLER FOR SQLEXCEPTION                               
	     BEGIN
	        -- Would like to have used this method but custom MESSAGE_TEXT is limited to 70 chars
	     		--GET DIAGNOSTICS EXCEPTION 1 v_token = MESSAGE_TEXT;
	     		--SET v_errmsg= 'sqlstate:' || SQLSTATE || ' sqlcode:' ||CHAR(SQLCODE) || ' message:' || v_token; 
	        --RESIGNAL SQLSTATE '70000' SET MESSAGE_TEXT=v_errmsg;   
	        RESIGNAL;                                               
	     END;
		
		 DECLARE EXIT HANDLER FOR SQLSTATE '70001'                              
			BEGIN    
				SET err_stmt='VALUES (' || v_schema_name || '.SELECT_CAL_MESSAGE(''CRREW4586E''))';
				PREPARE s_err12 FROM err_stmt;
				BEGIN
					DECLARE CE12 CURSOR FOR s_err12;
					OPEN CE12;
					FETCH CE12 INTO v_errmsg;
					CLOSE CE12;
				END;                                                                                                                       
				RESIGNAL SQLSTATE '70001' SET MESSAGE_TEXT = v_errmsg; 
				ROLLBACK;                                                                
			END;

		-- stop if sqlstate 42710 table or sequence already exists
	   DECLARE EXIT HANDLER FOR SQLSTATE '42710'
			 BEGIN  
			  IF POSSTR(stmt,'CREATE TABLE')>0 THEN
				 	RESIGNAL SQLSTATE '70002' SET MESSAGE_TEXT = ''; 
				 	ELSE
				 	RESIGNAL SQLSTATE '70003' SET MESSAGE_TEXT = '';  
			 	END IF;                                                            
			 END;
		 
		 DECLARE EXIT HANDLER FOR SQLSTATE '70004'                              
			BEGIN
				SET err_stmt='VALUES (' || v_schema_name || '.SELECT_CAL_MESSAGE(''CRREW4587E''))';
				PREPARE s_err21 FROM err_stmt;
				BEGIN
					DECLARE CE21 CURSOR FOR s_err21;
					OPEN CE21;
					FETCH CE21 INTO v_errmsg;
					CLOSE CE21;
				END;                                                                                                                       
				RESIGNAL SQLSTATE '70004' SET MESSAGE_TEXT = v_errmsg;
				ROLLBACK;                                                                  
			END;
		 
		 DECLARE EXIT HANDLER FOR SQLSTATE '70005'                              
			BEGIN
			 	SET err_stmt='VALUES (' || v_schema_name || '.SELECT_CAL_MESSAGE(''CRREW4588E''))';
				PREPARE s_err31 FROM err_stmt;
				BEGIN
					DECLARE CE31 CURSOR FOR s_err31;
					OPEN CE31;
					FETCH CE31 INTO v_errmsg;
					CLOSE CE31;
				END;                                                                                                                         
				RESIGNAL SQLSTATE '70005' SET MESSAGE_TEXT = v_errmsg;    
				ROLLBACK;                                                              
			END;
			
			DECLARE EXIT HANDLER FOR SQLSTATE '70006'                              
			BEGIN
			 	SET err_stmt='VALUES (' || v_schema_name || '.SELECT_CAL_MESSAGE(''CRREW4589E''))';
				PREPARE s_err41 FROM err_stmt;
				BEGIN
					DECLARE CE41 CURSOR FOR s_err41;
					OPEN CE41;
					FETCH CE41 INTO v_errmsg;
					CLOSE CE41;
				END;                                                                                                                                 
				RESIGNAL SQLSTATE '70006' SET MESSAGE_TEXT = v_errmsg;   
				ROLLBACK;                                                               
			END;
			
			DECLARE EXIT HANDLER FOR SQLSTATE '70007'                              
			BEGIN
				SET err_stmt='VALUES (' || v_schema_name || '.SELECT_CAL_MESSAGE(''CRREW4590E''))';
				PREPARE s_err5 FROM err_stmt;
				BEGIN
					DECLARE CE5 CURSOR FOR s_err5;
					OPEN CE5;
					FETCH CE5 INTO v_errmsg;
					CLOSE CE5;
				END;                                                                                                                           
				SIGNAL SQLSTATE '70007' SET MESSAGE_TEXT = v_errmsg; 
				ROLLBACK;                                                                  
			END;
			
			DECLARE EXIT HANDLER FOR SQLSTATE '70008'                              
			BEGIN
			    SET err_stmt='VALUES (' || v_schema_name || '.SELECT_CAL_MESSAGE(''CRREW4591E''))';
				PREPARE s_err51 FROM err_stmt;
				BEGIN
					DECLARE CE51 CURSOR FOR s_err51;
					OPEN CE51;
					FETCH CE51 INTO v_errmsg;
					CLOSE CE51;
				END;                                                                                                                    
				SIGNAL SQLSTATE '70008' SET MESSAGE_TEXT = v_errmsg;
				ROLLBACK;                                                                   
			END;
			
			DECLARE EXIT HANDLER FOR SQLSTATE '70009'                              
			BEGIN
			    SET err_stmt='VALUES (' || v_schema_name || '.SELECT_CAL_MESSAGE(''CRREW4592E''))';
				PREPARE s_err6 FROM err_stmt;
				BEGIN
					DECLARE CE6 CURSOR FOR s_err6;
					OPEN CE6;
					FETCH CE6 INTO v_errmsg;
					CLOSE CE6;
				END;                                                                                                                        
				SIGNAL SQLSTATE '70009' SET MESSAGE_TEXT = v_errmsg;      
				ROLLBACK;                                                             
			END;
			
			DECLARE EXIT HANDLER FOR SQLSTATE '70010'                              
			BEGIN
			    SET err_stmt='VALUES (' || v_schema_name || '.SELECT_CAL_MESSAGE(''CRREW4593E''))';
				PREPARE s_err7 FROM err_stmt;
				BEGIN
					DECLARE CE7 CURSOR FOR s_err7;
					OPEN CE7;
					FETCH CE7 INTO v_errmsg;
					CLOSE CE7;
				END;                                                                                                                    
				SIGNAL SQLSTATE '70010' SET MESSAGE_TEXT = v_errmsg; 
				ROLLBACK;                                                                  
			END;
		  
		  DECLARE EXIT HANDLER FOR SQLSTATE '70011'                              
			BEGIN 
			    SET err_stmt='VALUES (' || v_schema_name || '.SELECT_CAL_MESSAGE(''CRREW4594E''))';
				PREPARE s_err8 FROM err_stmt;
				BEGIN
					DECLARE CE8 CURSOR FOR s_err8;
					OPEN CE8;
					FETCH CE8 INTO v_errmsg;
					CLOSE CE8;
				END;                                                                                                                    
				SIGNAL SQLSTATE '70011' SET MESSAGE_TEXT = v_errmsg;  
				ROLLBACK;                                                                
			END;
				
		SET v_schema_name=UPPER(RTRIM(SCHEMA_NAME));
		
		IF (START_YEAR IS NULL) OR (START_YEAR>END_YEAR) THEN
			SIGNAL SQLSTATE '70004';
		END IF;
		
		IF (END_YEAR IS NULL) OR (END_YEAR < START_YEAR) THEN
			SIGNAL SQLSTATE '70005';
		END IF;
		
                --IF v_calendar_type<>1 AND v_calendar_type <> 2 THEN
		--		 SIGNAL SQLSTATE '70001'; 
		--END IF;    
		
		IF (length(rtrim(v_schema_name))=0) THEN
				 SIGNAL SQLSTATE '70006'; 
		END IF; 
		
		
		SET v_min_year=START_YEAR;
    SET v_max_year=END_YEAR;   
		
		SET stmt= 'SELECT COUNT(*) FROM ' || v_schema_name || '.D_DATE WITH UR';
    PREPARE st12 FROM stmt;
				BEGIN
					DECLARE CS12 CURSOR FOR st12;
					OPEN CS12;
					FETCH CS12 INTO v_total_rows;
					CLOSE CS12;
				END;
    
    IF (v_total_rows<>0) THEN
				 SET stmt='SELECT MIN(PERIOD_START_DATE), MAX(PERIOD_START_DATE) FROM ' || v_schema_name || '.D_DATE WITH UR';
								
								PREPARE st13 FROM stmt;
								BEGIN
									DECLARE CS2 CURSOR FOR st13;
									OPEN CS2;
									FETCH CS2 INTO v_min_date,v_max_date;
									CLOSE CS2;
							  END;
							  
							  
							  IF (NOT(DAY(v_min_date)=1 AND MONTH(v_min_date)=1)) THEN
							  		 SIGNAL SQLSTATE '70007'; 
						    END IF; 
						    
						    IF (NOT(DAY(v_max_date)=31 AND MONTH(v_max_date)=12)) THEN
							  		 SIGNAL SQLSTATE '70008'; 
						    END IF;

						  -- SET stmt='SELECT INT_VALUE FROM CONFIG.SYSCONFIG_INFO WHERE NAME=''CALENDAR_WEEK_TYPE''';
						   
						  -- PREPARE st2 FROM stmt;
						--		BEGIN
						--			DECLARE CS3 CURSOR FOR st2;
						--			OPEN CS3;
						--			FETCH CS3 INTO v_calendar_type;
						--			CLOSE CS3;
                                                 -- END;
               
						    --backward extension
							  IF (YEAR(v_min_date)> v_min_year) THEN 
							  	IF (YEAR(v_min_date)> v_max_year) THEN
							  		SIGNAL SQLSTATE '70010';
							  	ELSE
							  	
							  	SET stmt='CALL ' ||v_schema_name|| '.POPULATE_D_DATE(' || CHAR(v_min_year) || ',YEAR(''' || CHAR(v_min_date) || ''')-1,'''|| v_schema_name||''')';
									PREPARE s41 FROM stmt;
        		      EXECUTE s41;
        		      END IF;
							END IF;
								
								--FORWARD extension
							  IF (YEAR(v_max_date)< v_max_year) THEN  
							  	IF (YEAR(v_max_date)< (v_min_year-1)) THEN
							  	SIGNAL SQLSTATE '70011';
							  ELSE
							  	SET stmt='CALL ' ||v_schema_name|| '.POPULATE_D_DATE(YEAR(''' || CHAR(v_max_date) || ''')+1,' || CHAR(v_max_year) || ','''|| v_schema_name||''')';
									PREPARE s5 FROM stmt;
        		      EXECUTE s5;
        		    END IF;
        		 
							END IF;
							
        		  SET stmt2='CALL ' ||v_schema_name|| '.RECALCULATE_D_DATE (''' ||CHAR(v_schema_name)||''')';
        		  PREPARE s6 FROM stmt2;
        		  EXECUTE s6;		
			END IF;
			RETURN 0;  
END@  			

CREATE FUNCTION SELECT_ETL_DATE (ETLGRANULARITY INTEGER, ETLDATE TIMESTAMP,CAL_TYPE INTEGER) 
RETURNS INTEGER 
NO EXTERNAL ACTION 
READS SQL DATA
BEGIN ATOMIC
     DECLARE v_date_id INTEGER DEFAULT -1;
	 DECLARE v_cal_type CHAR(1) DEFAULT '';
	 DECLARE stmt VARCHAR(1000) DEFAULT ''; 
	 
	 SET v_cal_type=CASE WHEN CAL_TYPE =1 THEN 'G' ELSE 'I' END;
	  
	SET v_date_id= CASE ETLGRANULARITY WHEN 1 THEN (SELECT DATE_ID FROM D_DATE WHERE PERIOD_START_DATE= DATE(ETLDATE)) 
	WHEN 2 THEN (SELECT DATE_ID FROM D_WEEK WHERE  WEEK_OF_YEAR=(CASE WHEN v_cal_type='I' THEN WEEK_ISO(ETLDATE) ELSE WEEK(ETLDATE) END) AND YEAR_OF=YEAR(ETLDATE))
	WHEN 3 THEN (SELECT DATE_ID FROM D_MONTH WHERE MONTH_OF_YEAR=MONTH(ETLDATE) AND YEAR_OF=YEAR(ETLDATE)) END;
	  

	 
    RETURN v_date_id;
	 
END@